@@ -129,11 +129,12 @@
  *		This can be much slower and no memory statistics will be
  *		provided.
  */
-#if defined(PURIFY) || defined(MALLOC_TRACE)
-# define SYSMALLOC
-#else
-# undef SYSMALLOC
-#endif
+/* #if defined(PURIFY) || defined(MALLOC_TRACE) */
+#define SYSMALLOC
+/* #else
+ * # undef SYSMALLOC
+ * #endif
+ */
 
 /*
  * REMOTEHOST	Try to determine the remote host that we logged in from
