@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.exec.c,v 3.24 1993/07/07 19:16:17 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.06/RCS/sh.exec.c,v 3.34 1995/04/16 19:15:53 christos Exp $ */
 /*
  * sh.exec.c: Search, find, and execute a command!
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.exec.c,v 3.24 1993/07/07 19:16:17 christos Exp $")
+RCSID("$Id: sh.exec.c,v 3.34 1995/04/16 19:15:53 christos Exp $")
 
 #include "tc.h"
 #include "tw.h"
@@ -87,8 +87,8 @@ static Char *expath;		/* Path for exerr */
 # define HSHMUL		241
 
 static unsigned long *xhash = NULL;
-static unsigned int hashlength = 0;
-static unsigned int hashwidth = 0;
+static unsigned int hashlength = 0, uhashlength = 0;
+static unsigned int hashwidth = 0, uhashwidth = 0;
 static int hashdebug = 0;
 
 # define hash(a, b)	(((a) * HSHMUL + (b)) % (hashlength))
@@ -412,7 +412,15 @@ texec(sf, st)
     }
     if (errno == 0)
 #endif /* apollo */
-    (void) execv(f, t);
+    {
+#ifdef ISC_POSIX_EXEC_BUG
+	__setostype(0);		/* "0" is "__OS_SYSV" in <sys/user.h> */
+#endif /* ISC_POSIX_EXEC_BUG */
+	(void) execv(f, t);
+#ifdef ISC_POSIX_EXEC_BUG
+	__setostype(1);		/* "1" is "__OS_POSIX" in <sys/user.h> */
+#endif /* ISC_POSIX_EXEC_BUG */
+    }
 #ifdef VFORK
     Vt = 0;
 #endif /* VFORK */
@@ -453,7 +461,7 @@ texec(sf, st)
 	    v = adrof1(STRshell, &aliases);
 	    if (v == 0) {
 		vp = lastsh;
-		vp[0] = adrof(STRshell) ? value(STRshell) : STR_SHELLPATH;
+		vp[0] = adrof(STRshell) ? varval(STRshell) : STR_SHELLPATH;
 		vp[1] = NULL;
 #ifdef _PATH_BSHELL
 		if (fd != -1 
@@ -487,7 +495,13 @@ texec(sf, st)
 #ifdef VFORK
 	Vt = t;
 #endif /* VFORK */
+#ifdef ISC_POSIX_EXEC_BUG
+	__setostype(0);		/* "0" is "__OS_SYSV" in <sys/user.h> */
+#endif /* ISC_POSIX_EXEC_BUG */
 	(void) execv(f, t);
+#ifdef ISC_POSIX_EXEC_BUG
+	__setostype(1);		/* "1" is "__OS_POSIX" in <sys/user.h> */
+#endif /* ISC_POSIX_EXEC_BUG */
 #ifdef VFORK
 	Vt = 0;
 #endif /* VFORK */
@@ -546,7 +560,7 @@ execash(t, kp)
      * Hmm, we don't really want to do that now because we might
      * fail, but what is the choice
      */
-    rechist(NULL);
+    rechist(NULL, adrof(STRsavehist) != NULL);
 
 
     osigint  = signal(SIGINT, parintr);
@@ -644,36 +658,42 @@ dohash(vv, c)
     Char  **pv;
     int hashval;
 
-     USE(c);
+    USE(c);
 #ifdef FASTHASH
     if (vv && vv[1]) {
-        hashlength = atoi(short2str(vv[1]));
+        uhashlength = atoi(short2str(vv[1]));
         if (vv[2]) {
-	    hashwidth = atoi(short2str(vv[2]));
-	    if ((hashwidth != sizeof(unsigned char)) && 
-	        (hashwidth != sizeof(unsigned short)) && 
-	        (hashwidth != sizeof(unsigned long)))
-	        hashwidth = 0;
+	    uhashwidth = atoi(short2str(vv[2]));
+	    if ((uhashwidth != sizeof(unsigned char)) && 
+	        (uhashwidth != sizeof(unsigned short)) && 
+	        (uhashwidth != sizeof(unsigned long)))
+	        uhashwidth = 0;
 	    if (vv[3])
 		hashdebug = atoi(short2str(vv[3]));
         }
     }
 
-    if (hashwidth == 0) {
-        for (pv = v->vec; *pv; pv++, hashwidth++)
+    if (uhashwidth)
+	hashwidth = uhashwidth;
+    else {
+	hashwidth = 0;
+	for (pv = v->vec; *pv; pv++, hashwidth++)
 	    continue;
-        if (hashwidth <= widthof(unsigned char))
+	if (hashwidth <= widthof(unsigned char))
 	    hashwidth = sizeof(unsigned char);
-        else if (hashwidth <= widthof(unsigned short))
+	else if (hashwidth <= widthof(unsigned short))
 	    hashwidth = sizeof(unsigned short);
-        else if (hashwidth <= widthof(unsigned int))
+	else if (hashwidth <= widthof(unsigned int))
 	    hashwidth = sizeof(unsigned int);
 	else
 	    hashwidth = sizeof(unsigned long);
     }
-    if (hashlength == 0) 
-        hashlength = hashwidth * (8*64);/* "average" files per dir in path */
 
+    if (uhashlength)
+	hashlength = uhashlength;
+    else
+        hashlength = hashwidth * (8*64);/* "average" files per dir in path */
+    
     if (xhash)
         xfree((ptr_t) xhash);
     xhash = (unsigned long *) xcalloc((size_t) (hashlength * hashwidth), 
@@ -709,7 +729,7 @@ dohash(vv, c)
 	    hashval = hashname(str2short(dp->d_name));
 	    bis(hashval, i);
 	    if (hashdebug & 1)
-	        xprintf("hash=%-4d dir=%-2d prog=%s\n",
+	        xprintf(CGETS(13, 1, "hash=%-4d dir=%-2d prog=%s\n"),
 		        hashname(str2short(dp->d_name)), i, dp->d_name);
 #else /* OLD HASH */
 	    hashval = hash(hashname(str2short(dp->d_name)), i);
@@ -738,7 +758,6 @@ dounhash(v, c)
 #endif /* FASTHASH */
 }
 
-#ifdef VFORK
 /*ARGSUSED*/
 void
 hashstat(v, c)
@@ -749,17 +768,18 @@ hashstat(v, c)
     USE(v);
 #ifdef FASTHASH 
    if (havhash && hashlength && hashwidth)
-      xprintf("%d hash buckets of %d bits each\n",
+      xprintf(CGETS(13, 2, "%d hash buckets of %d bits each\n"),
 	      hashlength, hashwidth*8);
    if (hashdebug)
-      xprintf("debug mask = 0x%08x\n", hashdebug);
+      xprintf(CGETS(13, 3, "debug mask = 0x%08x\n"), hashdebug);
 #endif /* FASTHASH */
+#ifdef VFORK
    if (hits + misses)
-      xprintf("%d hits, %d misses, %d%%\n",
+      xprintf(CGETS(13, 4, "%d hits, %d misses, %d%%\n"),
 	      hits, misses, 100 * hits / (hits + misses));
+#endif
 }
 
-#endif /* VFORK */
 
 /*
  * Hash a command name.
@@ -865,15 +885,16 @@ executable(dir, name, dir_ok)
 	      access(strname, X_OK) == 0)));
 }
 
-void
-tellmewhat(lexp)
+int
+tellmewhat(lexp, str)
     struct wordent *lexp;
+    Char *str;
 {
     register int i;
     register struct biltins *bptr;
     register struct wordent *sp = lexp->next;
     bool    aliased = 0;
-    Char   *s0, *s1, *s2;
+    Char   *s0, *s1, *s2, *cmd;
     Char    qc;
 
     if (adrof1(sp->word, &aliases)) {
@@ -911,16 +932,23 @@ tellmewhat(lexp)
 
     for (bptr = bfunc; bptr < &bfunc[nbfunc]; bptr++) {
 	if (eq(sp->word, str2short(bptr->bname))) {
-	    if (aliased)
-		prlex(lexp);
-	    xprintf("%S: shell built-in command.\n", sp->word);
-	    flush();
+	    if (str == NULL) {
+		if (aliased)
+		    prlex(lexp);
+		xprintf(CGETS(13, 5, "%S: shell built-in command.\n"),
+			      sp->word);
+		flush();
+	    }
+	    else 
+		(void) Strcpy(str, sp->word);
 	    sp->word = s0;	/* we save and then restore this */
-	    return;
+	    return TRUE;
 	}
     }
 
-    if ((i = iscommand(strip(sp->word))) != 0) {
+    sp->word = cmd = globone(sp->word, G_IGNORE);
+
+    if ((i = iscommand(sp->word)) != 0) {
 	register Char **pv;
 	register struct varent *v;
 	bool    slash = any(short2str(sp->word), '/');
@@ -942,21 +970,33 @@ tellmewhat(lexp)
 	    else
 		prlex(lexp);
 	    sp->word = s0;	/* we save and then restore this */
-	    return;
+	    xfree((ptr_t) cmd);
+	    return TRUE;
 	}
 	s1 = Strspl(*pv, STRslash);
 	sp->word = Strspl(s1, sp->word);
 	xfree((ptr_t) s1);
-	prlex(lexp);
+	if (str == NULL)
+	    prlex(lexp);
+	else
+	    (void) Strcpy(str, sp->word);
 	xfree((ptr_t) sp->word);
     }
     else {
-	if (aliased)
-	    prlex(lexp);
-	xprintf("%S: Command not found.\n", sp->word);
-	flush();
+	if (str == NULL) {
+	    if (aliased)
+		prlex(lexp);
+	    xprintf(CGETS(13, 6, "%S: Command not found.\n"), sp->word);
+	    flush();
+	}
+	else {
+	    (void) Strcpy(str, sp->word);
+	    return FALSE;
+	}
     }
     sp->word = s0;		/* we save and then restore this */
+    xfree((ptr_t) cmd);
+    return TRUE;
 }
 
 /*
@@ -973,9 +1013,13 @@ dowhere(v, c)
     register Char **v;
     struct command *c;
 {
+    int found = 1;
     USE(c);
     for (v++; *v; v++)
-	(void) find_cmd(*v, 1);
+	found &= find_cmd(*v, 1);
+    /* Make status nonzero if any command is not found. */
+    if (!found)
+      set(STRstatus, Strsave(STR1), VAR_READWRITE);
 }
 
 int
@@ -990,15 +1034,15 @@ find_cmd(cmd, prt)
     int hashval, i, ex, rval = 0;
 
     if (prt && any(short2str(cmd), '/')) {
-	xprintf("where: / in command makes no sense\n");
-	return 0;
+	xprintf(CGETS(13, 7, "where: / in command makes no sense\n"));
+	return rval;
     }
 
     /* first, look for an alias */
 
     if (prt && adrof1(cmd, &aliases)) {
 	if ((var = adrof1(cmd, &aliases)) != NULL) {
-	    xprintf("%S is aliased to ", cmd);
+	    xprintf(CGETS(13, 8, "%S is aliased to "), cmd);
 	    blkpr(var->vec);
 	    xputchar('\n');
 	    rval = 1;
@@ -1011,15 +1055,16 @@ find_cmd(cmd, prt)
 	if (eq(cmd, str2short(bptr->bname))) {
 	    rval = 1;
 	    if (prt)
-		xprintf("%S is a shell built-in\n", cmd);
+		xprintf(CGETS(13, 9, "%S is a shell built-in\n"), cmd);
 	    else
-		return 1;
+		return rval;
 	}
     }
 
     /* last, look through the path for the command */
 
-    var = adrof(STRpath);
+    if ((var = adrof(STRpath)) == NULL)
+	return rval;
 
     hashval = havhash ? hashname(cmd) : 0;
 
@@ -1039,7 +1084,7 @@ find_cmd(cmd, prt)
 	ex = executable(*pv, sv, 0);
 #ifdef FASTHASH
 	if (!ex && (hashdebug & 2)) {
-	    xprintf("hash miss: ");
+	    xprintf(CGETS(13, 10, "hash miss: "));
 	    ex = 1;	/* Force printing */
 	}
 #endif /* FASTHASH */
@@ -1050,7 +1095,7 @@ find_cmd(cmd, prt)
 		xprintf("%S\n", cmd);
 	    }
 	    else
-		return 1;
+		return rval;
 	}
     }
     xfree((ptr_t) sv);
