@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.05/RCS/tc.bind.c,v 3.19 1994/06/18 19:48:50 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.06/RCS/tc.bind.c,v 3.24 1995/03/12 04:49:26 christos Exp $ */
 /*
  * tc.bind.c: Key binding functions
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.bind.c,v 3.19 1994/06/18 19:48:50 christos Exp $")
+RCSID("$Id: tc.bind.c,v 3.24 1995/03/12 04:49:26 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"
@@ -49,10 +49,11 @@ static	int    parsekey		__P((Char **));
 static	void   pkeys		__P((int, int));
 #endif /* OBSOLETE */
 
-static	void   printkey		__P((KEYCMD *, Char *));
+static	void   printkey		__P((KEYCMD *, CStr *));
 static	KEYCMD parsecmd		__P((Char *));
-static	Char  *parsestring	__P((Char *, Char *));
-static	Char  *parsebind	__P((Char *, Char *));
+static  void   bad_spec		__P((Char *));
+static	CStr  *parsestring	__P((Char *, CStr *));
+static	CStr  *parsebind	__P((Char *, CStr *));
 static	void   print_all_keys	__P((void));
 static	void   printkeys	__P((KEYCMD *, int, int));
 static	void   bindkey_usage	__P((void));
@@ -73,11 +74,16 @@ dobindkey(v, c)
     int     ntype, no, remove, key, bind;
     Char   *par;
     Char    p;
+    KEYCMD  cmd;
+    CStr    in;
+    CStr    out;
     Char    inbuf[200];
     Char    outbuf[200];
-    Char   *in;
-    Char   *out;
-    KEYCMD  cmd;
+    unsigned char ch;
+    in.buf = inbuf;
+    out.buf = outbuf;
+    in.len = 0;
+    out.len = 0;
 
     USE(c);
     if (!MapsAreInited)
@@ -88,8 +94,10 @@ dobindkey(v, c)
     key = remove = bind = 0;
     for (no = 1, par = v[no]; 
 	 par != NULL && (*par++ & CHAR) == '-'; no++, par = v[no]) {
-	if ((p = (*par & CHAR)) == '-')
+	if ((p = (*par & CHAR)) == '-') {
+	    no++;
 	    break;
+	}
 	else 
 	    switch (p) {
 	    case 'b':
@@ -139,42 +147,45 @@ dobindkey(v, c)
 
     if (key) {
 	if (!IsArrowKey(v[no]))
-	    xprintf("Invalid key name `%S'\n", v[no]);
-	in = v[no++];
+	    xprintf(CGETS(20, 1, "Invalid key name `%S'\n"), v[no]);
+	in.buf = v[no++];
+	in.len = Strlen(in.buf);
     }
     else {
 	if (bind) {
-	    if ((in = parsebind(v[no++], inbuf)) == NULL)
+	    if (parsebind(v[no++], &in) == NULL)
 		return;
 	}
 	else {
-	    if ((in = parsestring(v[no++], inbuf)) == NULL)
+	    if (parsestring(v[no++], &in) == NULL)
 		return;
 	}
     }
 
+    ch = (unsigned char) in.buf[0];
+
     if (remove) {
 	if (key) {
-	    (void) ClearArrowKeys(in);
+	    (void) ClearArrowKeys(&in);
 	    return;
 	}
-	if (in[1]) {
-	    (void) DeleteXkey(in);
+	if (in.len > 1) {
+	    (void) DeleteXkey(&in);
 	}
-	else if (map[(unsigned char) *in] == F_XKEY) {
-	    (void) DeleteXkey(in);
-	    map[(unsigned char) *in] = F_UNASSIGNED;
+	else if (map[ch] == F_XKEY) {
+	    (void) DeleteXkey(&in);
+	    map[ch] = F_UNASSIGNED;
 	}
 	else {
-	    map[(unsigned char) *in] = F_UNASSIGNED;
+	    map[ch] = F_UNASSIGNED;
 	}
 	return;
     }
     if (!v[no]) {
 	if (key)
-	    PrintArrowKeys(in);
+	    PrintArrowKeys(&in);
 	else
-	    printkey(map, in);
+	    printkey(map, &in);
 	return;
     }
     if (v[no + 1]) {
@@ -184,29 +195,29 @@ dobindkey(v, c)
     switch (ntype) {
     case XK_STR:
     case XK_EXE:
-	if ((out = parsestring(v[no], outbuf)) == NULL)
+	if (parsestring(v[no], &out) == NULL)
 	    return;
 	if (key) {
-	    if (SetArrowKeys(in, XmapStr(out), ntype) == -1)
-		xprintf("Bad key name: %S\n", in);
+	    if (SetArrowKeys(&in, XmapStr(&out), ntype) == -1)
+		xprintf(CGETS(20, 2, "Bad key name: %S\n"), in);
 	}
 	else
-	    AddXkey(in, XmapStr(out), ntype);
-	map[(unsigned char) *in] = F_XKEY;
+	    AddXkey(&in, XmapStr(&out), ntype);
+	map[ch] = F_XKEY;
 	break;
     case XK_CMD:
 	if ((cmd = parsecmd(v[no])) == 0)
 	    return;
 	if (key)
-	    (void) SetArrowKeys(in, XmapCmd((int) cmd), ntype);
+	    (void) SetArrowKeys(&in, XmapCmd((int) cmd), ntype);
 	else {
-	    if (in[1]) {
-		AddXkey(in, XmapCmd((int) cmd), ntype);
-		map[(unsigned char) *in] = F_XKEY;
+	    if (in.len > 1) {
+		AddXkey(&in, XmapCmd((int) cmd), ntype);
+		map[ch] = F_XKEY;
 	    }
 	    else {
-		ClearXkey(map, in);
-		map[(unsigned char) *in] = cmd;
+		ClearXkey(map, &in);
+		map[ch] = cmd;
 	    }
 	}
 	break;
@@ -221,15 +232,15 @@ dobindkey(v, c)
 static void
 printkey(map, in)
     KEYCMD *map;
-    Char   *in;
+    CStr   *in;
 {
     unsigned char outbuf[100];
     register struct KeyFuncs *fp;
 
-    if (in[0] == 0 || in[1] == 0) {
+    if (in->len < 2) {
 	(void) unparsestring(in, outbuf, STRQQ);
 	for (fp = FuncNames; fp->name; fp++) {
-	    if (fp->func == map[(unsigned char) *in]) {
+	    if (fp->func == map[(unsigned char) *(in->buf)]) {
 		xprintf("%s\t->\t%s\n", outbuf, fp->name);
 	    }
 	}
@@ -249,163 +260,101 @@ parsecmd(str)
 	    return (KEYCMD) fp->func;
 	}
     }
-    xprintf("Bad command name: %S\n", str);
+    xprintf(CGETS(20, 3, "Bad command name: %S\n"), str);
     return 0;
 }
 
-static Char *
-parsebind(s, b)
-    Char *s;
-    Char *b;
+
+static void
+bad_spec(str)
+    Char *str;
 {
-    static const char badspec[] = "Bad key spec %S\n";
-    Char *buf = b;
+    xprintf(CGETS(20, 4, "Bad key spec %S\n"), str);
+}
+
+static CStr *
+parsebind(s, str)
+    Char *s;
+    CStr *str;
+{
+    Char *b = str->buf;
+
+    if (Iscntrl(*s)) {
+	*b++ = *s;
+	*b = '\0';
+	str->len = b - str->buf;
+	return str;
+    }
+
     switch (*s) {
     case '^':
 	s++;
 	*b++ = (*s == '?') ? '\177' : ((*s & CHAR) & 0237);
 	*b = '\0';
-	return buf;
+	break;
 
     case 'F':
     case 'M':
     case 'X':
     case 'C':
 	if (s[1] != '-' || s[2] == '\0') {
-	    xprintf(badspec, s);
+	    bad_spec(s);
 	    return NULL;
 	}
 	s += 2;
 	switch (s[-2]) {
 	case 'F': case 'f':	/* Turn into ^[str */
 	    *b++ = '\033';
-	    Strcpy(b, s);
-	    return buf;
+	    while ((*b++ = *s++) != '\0')
+		continue;
+	    break;
 
 	case 'C': case 'c':	/* Turn into ^c */
 	    *b++ = (*s == '?') ? '\177' : ((*s & CHAR) & 0237);
 	    *b = '\0';
-	    return buf;
+	    break;
 
 	case 'X' : case 'x':	/* Turn into ^Xc */
 	    *b++ = 'X' & 0237;
 	    *b++ = *s;
 	    *b = '\0';
-	    return buf;
+	    break;
 
 	case 'M' : case 'm':	/* Turn into 0x80|c */
 	    *b++ = *s | 0x80;
 	    *b = '\0';
-	    return buf;
+	    break;
 
 	default:
 	    abort();
 	    /*NOTREACHED*/
 	    return NULL;
 	}
+	break;
 
     default:
-	xprintf(badspec, s);
+	bad_spec(s);
 	return NULL;
     }
+
+    str->len = b - str->buf;
+    return str;
 }
 
 
-int
-parseescape(ptr)
-    Char  **ptr;
-{
-    Char   *p, c;
-
-    p = *ptr;
-
-    if ((p[1] & CHAR) == 0) {
-	xprintf("Something must follow: %c\n", *p);
-	return -1;
-    }
-    if ((*p & CHAR) == '\\') {
-	p++;
-	switch (*p & CHAR) {
-	case 'a':
-	    c = '\007';		/* Bell */
-	    break;
-	case 'b':
-	    c = '\010';		/* Backspace */
-	    break;
-	case 't':
-	    c = '\011';		/* Horizontal Tab */
-	    break;
-	case 'n':
-	    c = '\012';		/* New Line */
-	    break;
-	case 'v':
-	    c = '\013';		/* Vertical Tab */
-	    break;
-	case 'f':
-	    c = '\014';		/* Form Feed */
-	    break;
-	case 'r':
-	    c = '\015';		/* Carriage Return */
-	    break;
-	case 'e':
-	    c = '\033';		/* Escape */
-	    break;
-	case '0':
-	case '1':
-	case '2':
-	case '3':
-	case '4':
-	case '5':
-	case '6':
-	case '7':
-	    {
-		register int cnt, val, ch;
-
-		for (cnt = 0, val = 0; cnt < 3; cnt++) {
-		    ch = *p++ & CHAR;
-		    if (ch < '0' || ch > '7') {
-			p--;
-			break;
-		    }
-		    val = (val << 3) | (ch - '0');
-		}
-		if ((val & 0xffffff00) != 0) {
-		    xprintf("Octal constant does not fit in a char.\n");
-		    return 0;
-		}
-		c = (Char) val;
-		--p;
-	    }
-	    break;
-	default:
-	    c = *p;
-	    break;
-	}
-    }
-    else if ((*p & CHAR) == '^' && (Isalpha(p[1] & CHAR) || 
-				    strchr("@^_?\\|[{]}", p[1] & CHAR))) {
-	p++;
-	c = (*p == '?') ? '\177' : ((*p & CHAR) & 0237);
-    }
-    else
-	c = *p;
-    *ptr = p;
-    return (c);
-}
-
-static Char *
+static CStr *
 parsestring(str, buf)
     Char   *str;
-    Char   *buf;
+    CStr   *buf;
 {
     Char   *b;
     Char   *p;
     int    es;
 
-    b = buf;
+    b = buf->buf;
     if (*str == 0) {
-	xprintf("Null string specification\n");
-	return 0;
+	xprintf(CGETS(20, 5, "Null string specification\n"));
+	return NULL;
     }
 
     for (p = str; *p != 0; p++) {
@@ -419,61 +368,20 @@ parsestring(str, buf)
 	    *b++ = *p & CHAR;
     }
     *b = 0;
+    buf->len = b - buf->buf;
     return buf;
 }
 
-unsigned char *
-unparsestring(str, buf, sep)
-    Char   *str;
-    unsigned char *buf;
-    Char   *sep;
-{
-    unsigned char *b;
-    Char   *p;
-
-    b = buf;
-    *b++ = sep[0];
-    if (*str == 0) {
-	*b++ = '^';
-	*b++ = '@';
-	*b++ = sep[1];
-	*b++ = 0;
-	return buf;
-    }
-
-    for (p = str; *p != 0; p++) {
-	if (Iscntrl(*p)) {
-	    *b++ = '^';
-	    if (*p == '\177')
-		*b++ = '?';
-	    else
-		*b++ = *p | 0100;
-	}
-	else if (*p == '^' || *p == '\\') {
-	    *b++ = '\\';
-	    *b++ = *p;
-	}
-	else if (*p == ' ' || (Isprint(*p) && !Isspace(*p))) {
-	    *b++ = *p;
-	}
-	else {
-	    *b++ = '\\';
-	    *b++ = ((*p >> 6) & 7) + '0';
-	    *b++ = ((*p >> 3) & 7) + '0';
-	    *b++ = (*p & 7) + '0';
-	}
-    }
-    *b++ = sep[1];
-    *b++ = 0;
-    return buf;			/* should check for overflow */
-}
-
 static void
 print_all_keys()
 {
     int     prev, i;
+    CStr nilstr;
+    nilstr.buf = NULL;
+    nilstr.len = 0;
 
-    xprintf("Standard key bindings\n");
+
+    xprintf(CGETS(20, 6, "Standard key bindings\n"));
     prev = 0;
     for (i = 0; i < 256; i++) {
 	if (CcKeyMap[prev] == CcKeyMap[i])
@@ -483,7 +391,7 @@ print_all_keys()
     }
     printkeys(CcKeyMap, prev, i - 1);
 
-    xprintf("Alternative key bindings\n");
+    xprintf(CGETS(20, 7, "Alternative key bindings\n"));
     prev = 0;
     for (i = 0; i < 256; i++) {
 	if (CcAltMap[prev] == CcAltMap[i])
@@ -492,10 +400,10 @@ print_all_keys()
 	prev = i;
     }
     printkeys(CcAltMap, prev, i - 1);
-    xprintf("Multi-character bindings\n");
-    PrintXkey(STRNULL);	/* print all Xkey bindings */
-    xprintf("Arrow key bindings\n");
-    PrintArrowKeys(STRNULL);
+    xprintf(CGETS(20, 8, "Multi-character bindings\n"));
+    PrintXkey(NULL);	/* print all Xkey bindings */
+    xprintf(CGETS(20, 9, "Arrow key bindings\n"));
+    PrintArrowKeys(&nilstr);
 }
 
 static void
@@ -505,16 +413,22 @@ printkeys(map, first, last)
 {
     register struct KeyFuncs *fp;
     Char    firstbuf[2], lastbuf[2];
+    CStr fb, lb;
     unsigned char unparsbuf[10], extrabuf[10];
+    fb.buf = firstbuf;
+    lb.buf = lastbuf;
 
     firstbuf[0] = (Char) first;
     firstbuf[1] = 0;
     lastbuf[0] = (Char) last;
     lastbuf[1] = 0;
+    fb.len = 1;
+    lb.len = 1;
+
     if (map[first] == F_UNASSIGNED) {
 	if (first == last)
-	    xprintf("%-15s->  is undefined\n",
-		    unparsestring(firstbuf, unparsbuf, STRQQ));
+	    xprintf(CGETS(20, 10, "%-15s->  is undefined\n"),
+		    unparsestring(&fb, unparsbuf, STRQQ));
 	return;
     }
 
@@ -522,24 +436,24 @@ printkeys(map, first, last)
 	if (fp->func == map[first]) {
 	    if (first == last) {
 		xprintf("%-15s->  %s\n",
-			unparsestring(firstbuf, unparsbuf, STRQQ), fp->name);
+			unparsestring(&fb, unparsbuf, STRQQ), fp->name);
 	    }
 	    else {
 		xprintf("%-4s to %-7s->  %s\n",
-			unparsestring(firstbuf, unparsbuf, STRQQ),
-			unparsestring(lastbuf, extrabuf, STRQQ), fp->name);
+			unparsestring(&fb, unparsbuf, STRQQ),
+			unparsestring(&lb, extrabuf, STRQQ), fp->name);
 	    }
 	    return;
 	}
     }
     if (map == CcKeyMap) {
-	xprintf("BUG!!! %s isn't bound to anything.\n",
-		unparsestring(firstbuf, unparsbuf, STRQQ));
+	xprintf(CGETS(20, 11, "BUG!!! %s isn't bound to anything.\n"),
+		unparsestring(&fb, unparsbuf, STRQQ));
 	xprintf("CcKeyMap[%d] == %d\n", first, CcKeyMap[first]);
     }
     else {
-	xprintf("BUG!!! %s isn't bound to anything.\n",
-		unparsestring(firstbuf, unparsbuf, STRQQ));
+	xprintf(CGETS(20, 11, "BUG!!! %s isn't bound to anything.\n"),
+		unparsestring(&fb, unparsbuf, STRQQ));
 	xprintf("CcAltMap[%d] == %d\n", first, CcAltMap[first]);
     }
 }
@@ -547,21 +461,37 @@ printkeys(map, first, last)
 static void
 bindkey_usage()
 {
-    xprintf(
-	"Usage: bindkey [options] [--] [in-string [out-string | command]]\n");
-    xprintf("    -a   bind key in alternative key binding\n");
-    xprintf("    -b   accept symbolic key definitions\n");
-    xprintf("    -s   bind an out-string instead of a command\n");
-    xprintf("    -c   bind a unix-command instead of a command\n");
-    xprintf("    -v   initialized maps to default vi bindings\n");
-    xprintf("    -e   initialized maps to default emacs bindings\n");
-    xprintf("    -d   initialized maps to default bindings\n");
-    xprintf("    -l   list available functions with descriptions\n");
-    xprintf("    -r   remove the binding of in-string\n");
-    xprintf("    -k   bind arrow key with name in in-string\n");
-    xprintf(
-       "\nIn no out-string or command is given, the binding for in-string\n");
-    xprintf("is printed or all bindings if in-strings is not given.\n");
+    xprintf(CGETS(20, 12,
+	    "Usage: bindkey [options] [--] [KEY [COMMAND]]\n"));
+    xprintf(CGETS(20, 13,
+    	    "    -a   list or bind KEY in alternative key map\n"));
+    xprintf(CGETS(20, 14,
+	    "    -b   interpret KEY as a C-, M-, F- or X- key name\n"));
+    xprintf(CGETS(20, 15,
+            "    -s   interpret COMMAND as a literal string to be output\n"));
+    xprintf(CGETS(20, 16,
+            "    -c   interpret COMMAND as a builtin or external command\n"));
+    xprintf(CGETS(20, 17,
+	    "    -v   bind all keys to vi bindings\n"));
+    xprintf(CGETS(20, 18,
+	    "    -e   bind all keys to emacs bindings\n"));
+    xprintf(CGETS(20, 19,
+	    "    -d   bind all keys to default editor's bindings\n"));
+    xprintf(CGETS(20, 20,
+	    "    -l   list editor commands with descriptions\n"));
+    xprintf(CGETS(20, 21,
+	    "    -r   remove KEY's binding\n"));
+    xprintf(CGETS(20, 22,
+	    "    -k   interpret KEY as a symbolic arrow-key name\n"));
+    xprintf(CGETS(20, 23,
+	    "    --   force a break from option processing\n"));
+    xprintf(CGETS(20, 24,
+	    "    -u   (or any invalid option) this message\n"));
+    xprintf("\n");
+    xprintf(CGETS(20, 25,
+	    "Without KEY or COMMAND, prints all bindings\n"));
+    xprintf(CGETS(20, 26,
+	    "Without COMMAND, prints the binding for KEY.\n"));
 }
 
 static void
@@ -570,7 +500,7 @@ list_functions()
     register struct KeyFuncs *fp;
 
     for (fp = FuncNames; fp->name; fp++) {
-	xprintf("%s\n          %s\n", fp->name, fp->description);
+	xprintf("%s\n          %s\n", fp->name, fp->desc);
     }
 }
 
@@ -717,11 +647,11 @@ parsekey(sp)
     KEYCMD  keycmd;
 
     if (s == NULL) {
-	xprintf("bad key specification -- null string\n");
+	xprintf(CGETS(20, 27, "bad key specification -- null string\n"));
 	return -1;
     }
     if (*s == 0) {
-	xprintf("bad key specification -- empty string\n");
+	xprintf(CGETS(20, 28, "bad key specification -- empty string\n"));
 	return -1;
     }
 
@@ -732,7 +662,8 @@ parsekey(sp)
 
     if ((s[0] == 'F' || s[0] == 'f') && s[1] == '-') {
 	if (s[2] == 0) {
-	    xprintf("Bad function-key specification.  Null key not allowed\n");
+	    xprintf(CGETS(20, 29,
+		   "Bad function-key specification.  Null key not allowed\n"));
 	    return (-1);
 	}
 	*sp = s + 2;
@@ -744,7 +675,8 @@ parsekey(sp)
 	for (s += 2; *s; s++) {	/* convert to hex; skip the first 0 */
 	    c *= 16;
 	    if (!Isxdigit(*s)) {
-		xprintf("bad key specification -- malformed hex number\n");
+		xprintf(CGETS(20, 30,
+			"bad key specification -- malformed hex number\n"));
 		return -1;	/* error */
 	    }
 	    if (Isdigit(*s))
@@ -759,7 +691,8 @@ parsekey(sp)
 	c = 0;
 	for (s++; *s; s++) {	/* convert to octal; skip the first 0 */
 	    if (!Isdigit(*s) || *s == '8' || *s == '9') {
-		xprintf("bad key specification -- malformed octal number\n");
+		xprintf(CGETS(20, 31,
+			"bad key specification -- malformed octal number\n"));
 		return -1;	/* error */
 	    }
 	    c = (c * 8) + *s - '0';
@@ -769,7 +702,8 @@ parsekey(sp)
 	c = 0;
 	for (; *s; s++) {	/* convert to octal; skip the first 0 */
 	    if (!Isdigit(*s)) {
-		xprintf("bad key specification -- malformed decimal number\n");
+		xprintf(CGETS(20, 32,
+		       "bad key specification -- malformed decimal number\n"));
 		return -1;	/* error */
 	    }
 	    c = (c * 10) + *s - '0';
@@ -805,8 +739,9 @@ parsekey(sp)
 
 	if (keycmd == F_XKEY) {
 	    if (*s == 0) {
-		xprintf("Bad function-key specification.\n");
-		xprintf("Null key not allowed\n");
+		xprintf(CGETS(20, 33,
+			      "Bad function-key specification.\n"));
+		xprintf(CGETS(20, 34, "Null key not allowed\n"));
 		return (-1);
 	    }
 	    *sp = s;
@@ -834,7 +769,8 @@ parsekey(sp)
 	    else if (!strcmp(ts, "delete"))
 		c = '\177';
 	    else {
-		xprintf("bad key specification -- unknown name \"%S\"\n", s);
+		xprintf(CGETS(20, 35,
+			"bad key specification -- unknown name \"%S\"\n", s));
 		return -1;	/* error */
 	    }
 	}
@@ -862,6 +798,7 @@ dobind(v, dummy)
     register struct KeyFuncs *fp;
     register int i, prev;
     Char   *p, *l;
+    CStr    cstr;
     Char    buf[1000];
 
     USE(dummy);
@@ -874,8 +811,8 @@ dobind(v, dummy)
 	ed_InitMaps();
 
     if (v[1] && v[2] && v[3]) {
-	xprintf(
-	   "usage: bind [KEY | COMMAND KEY | \"emacs\" | \"vi\" | \"-a\"]\n");
+	xprintf(CGETS(20, 36,
+	 "usage: bind [KEY | COMMAND KEY | \"emacs\" | \"vi\" | \"-a\"]\n"));
 	return;
     }
 
@@ -902,7 +839,9 @@ dobind(v, dummy)
 				*p++ = *l;
 			    }
 			    *p = 0;
-			    AddXkey(buf, XmapCmd(fp->func), XK_CMD);
+			    cstr.buf = buf;
+			    cstr.len = Strlen(buf);
+			    AddXkey(&cstr, XmapCmd(fp->func), XK_CMD);
 			}
 		    }
 		    return;
@@ -915,14 +854,18 @@ dobind(v, dummy)
 			    buf[0] = 033;
 			    buf[1] = c & ASCII;
 			    buf[2] = 0;
-			    AddXkey(buf, XmapCmd(fp->func), XK_CMD);
+			    cstr.buf = buf;
+			    cstr.len = Strlen(buf);
+			    AddXkey(&cstr, XmapCmd(fp->func), XK_CMD);
 			}
 		    }
 		    else {
 			buf[0] = 030;	/* ^X */
 			buf[1] = c & APOLLO_0377;
 			buf[2] = 0;
-			AddXkey(buf, XmapCmd(fp->func), XK_CMD);
+			cstr.buf = buf;
+			cstr.len = Strlen(buf);
+			AddXkey(&cstr, XmapCmd(fp->func), XK_CMD);
 			CcKeyMap[030] = F_XKEY;
 		    }
 		}
@@ -932,13 +875,15 @@ dobind(v, dummy)
 			buf[0] = 033;
 			buf[1] = c & ASCII;
 			buf[2] = 0;
-			AddXkey(buf, XmapCmd(fp->func), XK_CMD);
+			cstr.buf = buf;
+			cstr.len = Strlen(buf);
+			AddXkey(&cstr, XmapCmd(fp->func), XK_CMD);
 		    }
 		}
 		return;
 	    }
 	}
-	stderror(ERR_NAME | ERR_STRING, "Invalid function");
+	stderror(ERR_NAME | ERR_STRING, CGETS(20, 37, "Invalid function"));
     }
     else if (v[1]) {
 	char   *cv = short2str(v[1]);
@@ -975,7 +920,9 @@ dobind(v, dummy)
 	    if ((c = parsekey(&s)) == -1)
 		return;
 	    if (c == -2) {	/* extended key */
-		PrintXkey(s);
+		cstr.buf = s;
+		cstr.len = Strlen(s);
+		PrintXkey(&cstr);
 		return;
 	    }
 	    pkeys(c, c);	/* must be regular key */
@@ -998,7 +945,9 @@ dobind(v, dummy)
 	    prev = i;
 	}
 	pkeys(prev, i - 1);
-	PrintXkey(STRNULL);	/* print all Xkey bindings */
+	cstr.buf = NULL;
+	cstr.len = 0;
+	PrintXkey(&cstr);	/* print all Xkey bindings */
     }
     return;
 }
@@ -1024,7 +973,8 @@ pkeys(first, last)
     }
     if (map[first] == F_UNASSIGNED) {
 	if (first == last)
-	    xprintf(" %s\t\tis undefined\n", unparsekey(first | mask));
+	    xprintf(CGETS(20, 38, " %s\t\tis undefined\n"),
+		    unparsekey(first | mask));
 	return;
     }
 
@@ -1042,11 +992,12 @@ pkeys(first, last)
 	}
     }
     if (map == CcKeyMap) {
-	xprintf("BUG!!! %s isn't bound to anything.\n", unparsekey(first));
+	xprintf(CGETS(20, 11, "BUG!!! %s isn't bound to anything.\n"),
+		unparsekey(first));
 	xprintf("CcKeyMap[%d] == %d\n", first, CcKeyMap[first]);
     }
     else {
-	xprintf("BUG!!! %s isn't bound to anything.\n",
+	xprintf(CGETS(20, 11, "BUG!!! %s isn't bound to anything.\n"),
 		unparsekey(first & 0400));
 	xprintf("CcAltMap[%d] == %d\n", first, CcAltMap[first]);
     }
