@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.04/RCS/tc.decls.h,v 3.23 1993/06/24 15:29:37 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.06/RCS/tc.decls.h,v 3.35 1995/04/29 22:28:24 christos Exp $ */
 /*
  * tc.decls.h: Function declarations from all the tcsh modules
  */
@@ -58,9 +58,14 @@ extern	void		  showall	__P((Char **, struct command *));
  * tc.bind.c
  */
 extern	void		  dobindkey	__P((Char **, struct command *));
-extern	int		  parseescape	__P((Char **));
-extern	unsigned char    *unparsestring	__P((Char *, unsigned char *, Char *));
+#ifdef OBSOLETE
 extern	void		  dobind	__P((Char **, struct command *));
+#endif /* OBSOLETE */
+
+/*
+ * tc.defs.c:
+ */
+extern	void		  getmachine	__P((void));
 
 
 /*
@@ -80,6 +85,7 @@ extern	void		  dolist	__P((Char **, struct command *));
 extern	void		  dotelltc	__P((Char **, struct command *));
 extern	void		  doechotc	__P((Char **, struct command *));
 extern	void		  dosettc	__P((Char **, struct command *));
+extern	int		  cmd_expand	__P((Char *, Char *));
 extern	void		  dowhich	__P((Char **, struct command *));
 extern	struct process	 *find_stop_ed	__P((void));
 extern	void		  fg_proc_entry	__P((struct process *));
@@ -94,13 +100,18 @@ extern	void		  rmstar	__P((struct wordent *));
 extern	void		  continue_jobs	__P((struct wordent *));
 extern	Char		 *gettilde	__P((Char *));
 extern	Char		 *getusername	__P((Char **));
+#ifdef OBSOLETE
 extern	void		  doaliases	__P((Char **, struct command *));
+#endif /* OBSOLETE */
 extern	void		  shlvl		__P((int));
 extern	int		  fixio		__P((int, int));
 extern	int		  collate	__P((const Char *, const Char *));
 #ifdef HASHBANG
 extern	int		  hashbang	__P((int, Char ***));
 #endif /* HASHBANG */
+#ifdef REMOTEHOST
+extern	void		  remotehost	__P((void));
+#endif /* REMOTEHOST */
 
 
 /*
@@ -123,7 +134,11 @@ extern	void		  domigrate	__P((Char **, struct command *));
 extern	void 		  dowarp	__P((Char **, struct command *));
 #endif /* WARP */
 
-#ifdef masscomp
+#ifdef _CRAY
+extern	void 		  dodmmode	__P((Char **, struct command *));
+#endif /* _CRAY */
+
+#if defined(masscomp) || defined(hcx)
 extern	void		  douniverse	__P((Char **, struct command *));
 #endif /* masscomp */
 
@@ -152,10 +167,15 @@ extern	void	 	  fix_strcoll_bug	__P((void));
 extern	void	 	  osinit	__P((void));
 
 #ifdef NEEDmemmove
-extern void 		 *xmemmove	__P((ptr_t, const ptr_t, size_t));
+extern ptr_t 		 xmemmove	__P((ptr_t, const ptr_t, size_t));
 # define memmove(a, b, c) xmemmove((a), (b), (c))
 #endif /* NEEDmemmove */
 
+#ifdef NEEDmemset
+extern ptr_t 		 xmemset	__P((ptr_t, int, size_t));
+# define memset(a, b, c) xmemset((a), (b), (c))
+#endif /* NEEDmemset */
+
 
 #ifdef NEEDgetwd
 extern	char		 *xgetwd	__P((char *));
@@ -200,6 +220,7 @@ extern	pret_t		  xvsprintf	__P((char *, const char *, va_list));
 /*
  * tc.prompt.c
  */
+extern	void		  dateinit	__P((void));
 extern	void		  printprompt	__P((int, char *));
 extern	void		  tprintf	__P((int, Char *, Char *, size_t, 
 					     char *, time_t, ptr_t));
@@ -241,7 +262,7 @@ extern	void 		  sigpause	__P((int));
 extern	sigret_t	(*xsignal	__P((int, sigret_t (*)(int)))) ();
 # define signal(a, b)	  xsignal(a, b)
 #endif /* NEEDsignal */
-#if defined(_SEQUENT_) || (SYSVREL > 3 && defined(POSIXSIGS))
+#if defined(_SEQUENT_) || ((SYSVREL > 3 || defined(_DGUX_SOURCE)) && defined(POSIXSIGS)) || (defined(_AIX) && defined(POSIXSIGS))
 extern	sigmask_t	  sigsetmask	__P((sigmask_t));
 extern	sigmask_t	  sigblock	__P((sigmask_t));
 extern	void		  bsd_sigpause	__P((sigmask_t));
@@ -281,10 +302,9 @@ extern	char		 *short2qstr	__P((const Char *));
 
 
 /*
- * tc.vers.h:
+ * tc.vers.c:
  */
 extern	void		  fix_version	__P((void));
-extern	char		 *gethosttype	__P((void));
 
 /*
  * tc.who.c
@@ -292,9 +312,12 @@ extern	char		 *gethosttype	__P((void));
 #ifndef HAVENOUTMP
 extern	void		  initwatch	__P((void));
 extern	void		  resetwatch	__P((void));
-extern	void		  watch_login	__P((void));
-extern	char 		 *who_info	__P((ptr_t, int, char *));
+extern	void		  watch_login	__P((int));
+extern	const char 	 *who_info	__P((ptr_t, int, char *));
 extern	void		  dolog		__P((Char **, struct command *));
+# ifdef UTHOST
+extern	char		 *utmphost	__P((void));
+# endif /* UTHOST */
 #endif /* HAVENOUTMP */
 
 #endif /* _h_tc_decls */
