@@ -1868,7 +1868,7 @@ hashbang(fd, vp)
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <sys/socket.h>
-#include <sys/uio.h>	/* For struct iovec */
+#include <linux/uio.h>	/* For struct iovec */
 
 static sigret_t
 palarm(snum)
