@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.05/RCS/tc.os.c,v 3.36 1994/04/12 17:37:37 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.06/RCS/tc.os.c,v 3.41 1995/04/16 19:15:53 christos Exp $ */
 /*
  * tc.os.c: OS Dependent builtin functions
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.os.c,v 3.36 1994/04/12 17:37:37 christos Exp $")
+RCSID("$Id: tc.os.c,v 3.41 1995/04/16 19:15:53 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -120,11 +120,11 @@ dosetpath(arglist, c)
 
     /* note that npaths != 0 */
 
-    spaths = (char **) xmalloc(npaths * sizeof *spaths);
+    spaths = (char **) xmalloc((size_t) npaths * sizeof *spaths);
     setzero((char *) spaths, npaths * sizeof *spaths);
-    cpaths = (char **) xmalloc((npaths + 1) * sizeof *cpaths);
+    cpaths = (char **) xmalloc((size_t) (npaths + 1) * sizeof *cpaths);
     setzero((char *) cpaths, (npaths + 1) * sizeof *cpaths);
-    cmds = (char **) xmalloc((ncmds + 1) * sizeof *cmds);
+    cmds = (char **) xmalloc((size_t) (ncmds + 1) * sizeof *cmds);
     setzero((char *) cmds, (ncmds + 1) * sizeof *cmds);
     for (i = 0; i < npaths; i++) {
 	char   *val = getenv(short2str(pathvars[i]));
@@ -132,8 +132,8 @@ dosetpath(arglist, c)
 	if (val == NULL)
 	    val = "";
 
-	spaths[i] = xmalloc((Strlen(pathvars[i]) + strlen(val)
-			     + 2) * sizeof **spaths);
+	spaths[i] = (char *) xmalloc((size_t) (Strlen(pathvars[i]) +
+				      strlen(val) + 2) * sizeof **spaths);
 	(void) strcpy(spaths[i], short2str(pathvars[i]));
 	(void) strcat(spaths[i], "=");
 	(void) strcat(spaths[i], val);
@@ -145,7 +145,7 @@ dosetpath(arglist, c)
 
 	if (val == NULL)
 	    goto abortpath;
-	cmds[i] = xmalloc(Strlen(val) + 1);
+	cmds[i] = (char *) xmalloc((size_t) Strlen(val) + 1);
 	(void) strcpy(cmds[i], short2str(val));
     }
 
@@ -373,10 +373,10 @@ dosetspath(v, c)
 	    p[i] = st->sf_id;
 	else {
 	    setname(s);
-	    stderror(ERR_NAME | ERR_STRING, "Bad cpu/site name");
+	    stderror(ERR_NAME | ERR_STRING, CGETS(23, 1, "Bad cpu/site name"));
 	}
 	if (i == MAXSITE - 1)
-	    stderror(ERR_NAME | ERR_STRING, "Site path too long");
+	    stderror(ERR_NAME | ERR_STRING, CGETS(23, 2, "Site path too long"));
     }
     if (setspath(p, i) == -1)
 	stderror(ERR_SYSTEM, "setspath", strerror(errno));
@@ -394,9 +394,9 @@ sitename(pid)
     struct sf *st;
 
     if ((ss = site(pid)) == -1 || (st = sfnum(ss)) == NULL)
-	return ("unknown");
+	return CGETS(23, 3, "unknown");
     else
-	return (st->sf_sname);
+	return st->sf_sname;
 }
 
 static int
@@ -416,15 +416,16 @@ migratepid(pid, new_site)
 
     if (need_local) {
 	if ((new_site = site(0)) == -1) {
-	    xprintf("site: %s\n", strerror(errno));
+	    xprintf(CGETS(23, 4, "site: %s\n"), strerror(errno));
 	    return (-1);
 	}
 	if ((st = sfnum(new_site)) == NULL) {
-	    xprintf("%d: Site not found\n", new_site);
+	    xprintf(CGETS(23, 5, "%d: Site not found\n"), new_site);
 	    return (-1);
 	}
 	if (setlocal(st->sf_local, strlen(st->sf_local)) == -1) {
-	    xprintf("setlocal: %s: %s\n", st->sf_local, strerror(errno));
+	    xprintf(CGETS(23, 6, "setlocal: %s: %s\n"),
+			  st->sf_local, strerror(errno));
 	    return (-1);
 	}
     }
@@ -469,7 +470,7 @@ domigrate(v, c)
 	dont_free = 1;
 	if ((st = sfname(s)) == NULL) {
 	    setname(s);
-	    stderror(ERR_NAME | ERR_STRING, "Site not found");
+	    stderror(ERR_NAME | ERR_STRING, CGETS(23, 7, "Site not found"));
 	}
 	dont_free = 0;
 	new_site = st->sf_id;
@@ -527,6 +528,45 @@ done:
 
 #endif /* TCF */
 
+/***
+ *** CRAY ddmode <velo@sesun3.epfl.ch> (Martin Ouwehand EPFL-SIC/SE)
+ ***/
+#ifdef _CRAY
+void
+dodmmode(v, c)
+    Char  **v;
+    struct command *c;
+{
+    Char *cp = v[1];
+
+    USE(c);
+
+    if ( !cp ) {
+	int mode;
+
+	mode = dmmode(0);
+	dmmode(mode);
+	xprintf("%d\n",mode);
+    }
+    else {
+	if (cp[1] != '\0')
+	    stderror(ERR_DMMODE);
+	else
+	    switch(*cp) {
+	    case '0':
+		dmmode(0);
+		break;
+	    case '1':
+		dmmode(1);
+		break;
+	    default:
+		stderror(ERR_DMMODE);
+	    }
+    }
+}
+#endif /* _CRAY */
+
+
 /***
  *** CONVEX Warps.
  ***/
@@ -561,7 +601,7 @@ dowarp(v, c)
     if (setjmp(sigsys_buf)) {
 	signal(SIGSYS, old_sigsys_handler);
 	stderror(ERR_NAME | ERR_STRING, 
-		 "You're trapped in a universe you never made");
+		 CGETS(23, 8, "You're trapped in a universe you never made"));
 	return;
     }
     old_sigsys_handler = signal(SIGSYS, catch_sigsys);
@@ -571,8 +611,8 @@ dowarp(v, c)
     v++;
     if (*v == 0) {		/* display warp value */
 	if (warp < 0)
-	    stderror(ERR_NAME | ERR_STRING, "Getwarp failed");
-	we = getwarpbyvarval(warp);
+	    stderror(ERR_NAME | ERR_STRING, CGETS(23, 9, "Getwarp failed"));
+	we = getwarpbyvalue(warp);
 	if (we)
 	    printf("%s\n", we->w_name);
 	else
@@ -591,10 +631,10 @@ dowarp(v, c)
 		warp = -1;
 	}
 	if ((warp < 0) || (warp >= WARP_MAXLINK))
-	    stderror(ERR_NAME | ERR_STRING, "Invalid warp");
+	    stderror(ERR_NAME | ERR_STRING, CGETS(23, 10, "Invalid warp"));
 	if ((setwarp(warp) < 0) || (getwarp() != warp)) {
 	    (void) setwarp(oldwarp);
-	    stderror(ERR_NAME | ERR_STRING, "Setwarp failed");
+	    stderror(ERR_NAME | ERR_STRING, CGETS(23, 11, "Setwarp failed"));
 	}
     }
     signal(SIGSYS, old_sigsys_handler);
@@ -621,7 +661,7 @@ douniverse(v, c)
 	xprintf("%s\n", ubuf);
     }
     else if (*cp == '\0' || setuniverse(short2str(cp)) != 0)
-	stderror(ERR_NAME | ERR_STRING, "Illegal universe");
+	stderror(ERR_NAME | ERR_STRING, CGETS(23, 12, "Illegal universe"));
 }
 #endif /* masscomp || hcx */
 
@@ -753,6 +793,7 @@ fix_yp_bugs()
 {
     char   *mydomain;
 
+    extern int yp_get_default_domain();
     /*
      * PWP: The previous version assumed that yp domain was the same as the
      * internet name domain.  This isn't allways true. (Thanks to Mat Landau
@@ -816,7 +857,7 @@ osinit()
 #ifdef aiws
     {
 	struct sigstack inst;
-	inst.ss_sp = xmalloc(4192) + 4192;
+	inst.ss_sp = (char *) xmalloc((size_t) 4192) + 4192;
 	inst.ss_onstack = 0;
 	sigstack(&inst, NULL);
     }
@@ -837,7 +878,7 @@ xstrerror(i)
     if (i >= 0 && i < sys_nerr) {
 	return sys_errlist[i];
     } else {
-	(void) xsprintf(errbuf, "Unknown Error: %d", i);
+	(void) xsprintf(errbuf, CGETS(23, 13, "Unknown Error: %d"), i);
 	return errbuf;
     }
 }
@@ -860,11 +901,11 @@ xgethostname(name, namlen)
     retval = uname(&uts);
 
 #  ifdef DEBUG
-    xprintf("sysname:  %s\n", uts.sysname);
-    xprintf("nodename: %s\n", uts.nodename);
-    xprintf("release:  %s\n", uts.release);
-    xprintf("version:  %s\n", uts.version);
-    xprintf("machine:  %s\n", uts.machine);
+    xprintf(CGETS(23, 14, "sysname:  %s\n"), uts.sysname);
+    xprintf(CGETS(23, 15, "nodename: %s\n"), uts.nodename);
+    xprintf(CGETS(23, 16, "release:  %s\n"), uts.release);
+    xprintf(CGETS(23, 17, "version:  %s\n"), uts.version);
+    xprintf(CGETS(23, 18, "machine:  %s\n"), uts.machine);
 #  endif /* DEBUG */
     i = strlen(uts.nodename) + 1;
     (void) strncpy(name, uts.nodename, i < namlen ? i : namlen);
@@ -949,21 +990,23 @@ xgetwd(pathname)
 		if (d.st_ino == rino && d.st_dev == rdev && d.st_size == rsize)
 			break;		/* reached root directory */
 		if ((dirp = opendir("..")) == NULL) {
-        		(void) xsprintf(pathname,
-                        "getwd: Cannot open \"..\" (%s)", strerror(errno));
+        		(void) xsprintf(pathname, CGETS(23, 19,
+					"getwd: Cannot open \"..\" (%s)"),
+					strerror(errno));
 			goto fail;
 		}
 		if (chdir("..") < 0) {
-        		(void) xsprintf(pathname,
-                        "getwd: Cannot chdir to \"..\" (%s)", strerror(errno));
+        		(void) xsprintf(pathname, CGETS(23, 20,
+					"getwd: Cannot chdir to \"..\" (%s)"),
+					strerror(errno));
 			goto fail;
 		}
 		do {
 			if((dir = readdir(dirp)) == NULL) {
 				closedir(dirp);
-        			(void) xsprintf(pathname,
-                        	"getwd: Read error in \"..\" (%s)",
-				strerror(errno));
+        			(void) xsprintf(pathname, CGETS(23, 21,
+					    "getwd: Read error in \"..\" (%s)"),
+					    strerror(errno));
 				goto fail;
 			}
 			stat(dir->d_name, &dd);
@@ -980,9 +1023,9 @@ xgetwd(pathname)
 	else {
 		(void) strcpy(pathname, pnptr);
 		if (chdir(pnptr) < 0) {
-        		(void) xsprintf(pathname,
-                        "getwd: Cannot change back to \".\" (%s)",
-			strerror(errno));
+        		(void) xsprintf(pathname, CGETS(23, 22,
+				"getwd: Cannot change back to \".\" (%s)"),
+				strerror(errno));
 			return (NULL);
 		}
 	}
@@ -1031,8 +1074,9 @@ xgetwd(pathname)
 
     /* find the inode of root */
     if (stat("/", &st_root) == -1) {
-	(void) xsprintf(pathname,
-			"getwd: Cannot stat \"/\" (%s)", strerror(errno));
+	(void) xsprintf(pathname, CGETS(23, 23, 
+			"getwd: Cannot stat \"/\" (%s)"),
+			strerror(errno));
 	return (NULL);
     }
     pathbuf[MAXPATHLEN - 1] = '\0';
@@ -1042,8 +1086,9 @@ xgetwd(pathname)
 
     /* find the inode of the current directory */
     if (lstat(".", &st_cur) == -1) {
-	(void) xsprintf(pathname,
-			"getwd: Cannot stat \".\" (%s)", strerror(errno));
+	(void) xsprintf(pathname, CGETS(23, 24,
+					"getwd: Cannot stat \".\" (%s)"),
+			strerror(errno));
 	return (NULL);
     }
     nextpathptr = strrcpy(nextpathptr, "../");
@@ -1060,14 +1105,14 @@ xgetwd(pathname)
 
 	/* open the parent directory */
 	if (stat(nextpathptr, &st_dotdot) == -1) {
-	    (void) xsprintf(pathname,
-			    "getwd: Cannot stat directory \"%s\" (%s)",
+	    (void) xsprintf(pathname, CGETS(23, 25,
+			    "getwd: Cannot stat directory \"%s\" (%s)"),
 			    nextpathptr, strerror(errno));
 	    return (NULL);
 	}
 	if ((dp = opendir(nextpathptr)) == NULL) {
-	    (void) xsprintf(pathname,
-			    "getwd: Cannot open directory \"%s\" (%s)",
+	    (void) xsprintf(pathname, CGETS(23, 26,
+			    "getwd: Cannot open directory \"%s\" (%s)"),
 			    nextpathptr, strerror(errno));
 	    return (NULL);
 	}
@@ -1111,7 +1156,8 @@ xgetwd(pathname)
 	    }
 	}
 	if (d == NULL) {
-	    (void) xsprintf(pathname, "getwd: Cannot find \".\" in \"..\" (%s)",
+	    (void) xsprintf(pathname, CGETS(23, 27,
+			    "getwd: Cannot find \".\" in \"..\" (%s)"),
 			    strerror(save_errno ? save_errno : ENOENT));
 	    (void) closedir(dp);
 	    return (NULL);
@@ -1216,7 +1262,8 @@ getv(v)
     else if (eq(v, STRsys53))
 	return(0);
     else 
-	stderror(ERR_NAME | ERR_SYSTEM, short2str(v), "Invalid system type");
+	stderror(ERR_NAME | ERR_SYSTEM, short2str(v),
+		 CGETS(23, 28, "Invalid system type"));
     /*NOTREACHED*/
     return(0);
 }
@@ -1232,7 +1279,8 @@ dover(v, c)
     setname(short2str(*v++));
     if (!*v) {
 	if (!(p = tgetenv(STRSYSTYPE)))
-	    stderror(ERR_NAME | ERR_STRING, "System type is not set");
+	    stderror(ERR_NAME | ERR_STRING,
+		     CGETS(23, 29, "System type is not set"));
 	xprintf("%S\n", p);
     }
     else {
