@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.05/RCS/tc.prompt.c,v 3.21 1994/05/26 13:11:20 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.06/RCS/tc.prompt.c,v 3.23 1995/03/12 04:49:26 christos Exp $ */
 /*
  * tc.prompt.c: Prompt printing stuff
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.prompt.c,v 3.21 1994/05/26 13:11:20 christos Exp $")
+RCSID("$Id: tc.prompt.c,v 3.23 1995/03/12 04:49:26 christos Exp $")
 
 #include "ed.h"
 
@@ -49,9 +49,65 @@ RCSID("$Id: tc.prompt.c,v 3.21 1994/05/26 13:11:20 christos Exp $")
  *	   16-Feb-94 - rewrote directory prompt code, added $ellipsis
  */
 
-static char   *month_list[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
-			"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
-static char   *day_list[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
+static char   *month_list[12];
+static char   *day_list[7];
+
+void
+dateinit()
+{
+#ifdef notyet
+  int i;
+
+  setlocale(LC_TIME, "");
+
+  for (i = 0; i < 12; i++)
+      xfree((ptr_t) month_list[i]);
+  month_list[0] = strsave(_time_info->abbrev_month[0]);
+  month_list[1] = strsave(_time_info->abbrev_month[1]);
+  month_list[2] = strsave(_time_info->abbrev_month[2]);
+  month_list[3] = strsave(_time_info->abbrev_month[3]);
+  month_list[4] = strsave(_time_info->abbrev_month[4]);
+  month_list[5] = strsave(_time_info->abbrev_month[5]);
+  month_list[6] = strsave(_time_info->abbrev_month[6]);
+  month_list[7] = strsave(_time_info->abbrev_month[7]);
+  month_list[8] = strsave(_time_info->abbrev_month[8]);
+  month_list[9] = strsave(_time_info->abbrev_month[9]);
+  month_list[10] = strsave(_time_info->abbrev_month[10]);
+  month_list[11] = strsave(_time_info->abbrev_month[11]);
+
+  for (i = 0; i < 7; i++)
+      xfree((ptr_t) day_list[i]);
+  day_list[0] = strsave(_time_info->abbrev_wkday[0]);
+  day_list[1] = strsave(_time_info->abbrev_wkday[1]);
+  day_list[2] = strsave(_time_info->abbrev_wkday[2]);
+  day_list[3] = strsave(_time_info->abbrev_wkday[3]);
+  day_list[4] = strsave(_time_info->abbrev_wkday[4]);
+  day_list[5] = strsave(_time_info->abbrev_wkday[5]);
+  day_list[6] = strsave(_time_info->abbrev_wkday[6]);
+#else
+  month_list[0] = "Jan";
+  month_list[1] = "Feb";
+  month_list[2] = "Mar";
+  month_list[3] = "Apr";
+  month_list[4] = "May";
+  month_list[5] = "Jun";
+  month_list[6] = "Jul";
+  month_list[7] = "Aug";
+  month_list[8] = "Sep";
+  month_list[9] = "Oct";
+  month_list[10] = "Nov";
+  month_list[11] = "Dec";
+
+  day_list[0] = "Sun";
+  day_list[1] = "Mon";
+  day_list[2] = "Tue";
+  day_list[3] = "Wed";
+  day_list[4] = "Thu";
+  day_list[5] = "Fri";
+  day_list[6] = "Sat";
+#endif
+}
+
 void
 printprompt(promptno, str)
     int     promptno;
@@ -111,13 +167,13 @@ tprintf(what, buf, fmt, siz, str, tim, info)
     Char   *z, *q;
     Char    attributes = 0;
     static int print_prompt_did_ding = 0;
-    char   *cz;
+    const char   *cz;
     Char    buff[BUFSIZE];
     char    cbuff[BUFSIZE];
 
     Char *p  = buf;
     Char *ep = &p[siz];
-    Char *cp = fmt;
+    Char *cp = fmt, Scp;
     struct tm *t = localtime(&tim);
 
 			/* prompt stuff */
@@ -125,7 +181,6 @@ tprintf(what, buf, fmt, siz, str, tim, info)
     extern int tlength;	/* cache cleared */
     register int updirs;
     int pdirs;
-    Char    Scp;
 
     for (; *cp; cp++) {
 	if (p >= ep)
@@ -141,7 +196,7 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 			if (p >= ep) break;
 		break;
 	    case '#':
-		*p++ = attributes | ((uid == 0) ? '#' : '>');
+		*p++ = attributes | ((uid == 0) ? PRCHROOT : PRCH);
 		break;
 	    case '!':
 	    case 'h':
